#
# set up...
#
setwd("C:/Users/Owner/Desktop/week11")


#
# read in data
#
ab <- read.table("ab.txt", header=F)
ab <- ab[,c(14,15,16,17,18,19,20,21,25,28)]
names(ab) <- c("h","s","v","hstd","sstd","vstd","tx1","tx2","tx3","tx4")
View(ab)

be <- read.table("be.txt", header=F)
be <- be[,c(14,15,16,17,18,19,20,21,25,28)]
names(be) <- c("h","s","v","hstd","sstd","vstd","tx1","tx2","tx3","tx4")
View(be)

all <- data.frame( rbind(ab,be) )
View(all)

# divide by weights
all$h <- all$h / 255.0
all$s <- all$s / 255.0
all$v <- all$v / 255.0 # max

all$hstd <- all$hstd / 155.0
all$sstd <- all$sstd / 155.0
all$vstd <- all$vstd / 155.0 # max was about half that

all$tx1 <- all$tx1 / 4000.0 # about max
all$tx2 <- all$tx2 / 4000.0 # about max
all$tx3 <- all$tx3 / 40.0 # about max
all$tx4 <- all$tx4 / 8.0 # about max

# make labels - first 4092 are "above" next are "below"
label <- c( rep.int(0,4092), rep.int(1,4092) )
allL <- data.frame( cbind(all,label) )


# create some plots
coff <- 1
plot( allL$h, allL$tx3, col=(allL$label+coff), cex=.5)
legend(0.1,0.8,legend=c("above","below"),col=(c(0,1)+coff),pch=1,cex=1.0)

coff <- 1
plot( allL$hstd, allL$tx3, col=(allL$label+coff), cex=.5)
legend(0.1,0.8,legend=c("above","below"),col=(c(0,1)+coff),pch=1,cex=1.0)

coff <- 3
plot( allL$h, allL$s, col=(allL$label+coff), cex=.5)
legend(0.1,0.8,legend=c("above","below"),col=(c(0,1)+coff),pch=1,cex=1.0)

coff <- 3
plot( allL$hstd, allL$s, col=(allL$label+coff), cex=.5)
legend(0.1,0.8,legend=c("above","below"),col=(c(0,1)+coff),pch=1,cex=1.0)

coff <- 5
plot( allL$tx3, allL$tx4, col=(allL$label+coff), cex=.5)
legend(0.1,0.8,legend=c("above","below"),col=(c(0,1)+coff),pch=1,cex=1.0)


#
# unsupervised investigation
#

# unsupervised -- check clusters!
# we'd like to see two...
# try it!

# function "myplclust" for colored-leaf dendrograms
myplclust <- function( hclust, lab.col, hang=0.1, ... )
{
  ## modifiction of plclust for plotting hclust objects *in colour*!
  ## Copyright Eva KF Chan 2009
  ## Arguments:
  ## hclust: hclust object
  ## lab.col: colour for the labels; NA=default device foreground colour
  ## hang: as in hclust & plclust
  ## Side effect: A display of hierarchical cluster with coloured leaf labels. 
  y <- rep(hclust$height,2); x <- as.numeric(hclust$merge)
  y <- y[which(x<0)]; x <- x[which(x<0)]; x <- abs(x)
  y <- y[order(x)]; x <- x[order(x)]
  plot( hclust, labels=FALSE, hang=hang, ... )
  text( x=x, y=y[hclust$order]-(max(hclust$height)*hang),
        labels=hclust$order, col=lab.col[hclust$order], 
        srt=90, adj=c(1,0.5), xpd=NA, ... )
}


dists <- dist(all)
hClustering_s <- hclust(dists,method="single") 
hClustering_w <- hclust(dists,method="ward") 

# plotting the original points with cutree determining colors...
plot.new()
par(oma=c(2,2,2,2))
par(mai=c(1,1,1,1))
clusters <- cutree(hClustering_w,k=2)
myplclust(hClustering_w,lab.col=clusters)
myplclust(hClustering_s,lab.col=clusters)
#plot(x,y,col=cutree(hClustering_s,k=2),pch=19,cex=2)


# for "elbow" plot method of choosing # of centers
var_explained <- function(num_centers)
{
  km <- kmeans(all,centers=num_centers)
  plot(all$h,all$tx3,col=km$cluster,
       xlab="hue", ylab="texture tx3")
  var_expl <- km$betweenss/km$totss
  return(var_expl)
}

# demo for multiple centers...
run_n <- function()
{
  NUM_CENTERS_TESTED <- 7
  results <- vector(mode="numeric",length=7)
  for (num_centers in 1:NUM_CENTERS_TESTED)
  {
    var_expl <- var_explained(num_centers)
    results[num_centers] <- var_expl
  }
  
  # plot variance explained for each number of centers...
  plot(1:NUM_CENTERS_TESTED, results,
       type="b",pch=16,col="black",
       xlab="number of centers",ylab="variance explained")
}



#
# compare logistic linear regression vs. k-nearest neighbors
#



# now try lm vs knn with cross-validation...

data <- allL
data.train <- data  # resubstitution error
data.test <- data   # when train == test



# cross-validation set-up - linear
library(cvTools)
NUM_FOLDS <- 10
errors <- vector("numeric",NUM_FOLDS)  # container for errors
cvf <- cvFolds( nrow(data), K=NUM_FOLDS, type="consecutive" )
data <- data[sample(nrow(data)),]  # randomly permutes the rows
w <- cvf$which

# run the cross-validation
for (i in 1:NUM_FOLDS)
{
  data.train <- data[w!=i,]  # training data
  data.notrain <- data[w==i,]   # non-training data
  # not really necessary, but safe...
  data.test <- data.notrain[,-11]   # removes the predicted column
  
  actual_results <- data.notrain$label
  error <- model_and_check( data.train, data.test, actual_results )
  
  errors[i] <- error
  cat("error at fold ", i, " is ", error, "\n")
}

# plot the errors created
plot( errors, pch=19, col=4 )

cat("Mean error: ", mean(errors))

# builds a model and checks the results vs. actual results...
model_and_check <- function( data.train, data.test, actual_results )
{
  # build the model
  lm_mod <- glm(label ~ ., data=data.train, family=binomial(logit) )
  print(summary(lm_mod))
  # get the predicted results
  pr <- predict( lm_mod, newdata=data.test )
  # compute the error from predicted to actual
  prr <- ifelse( pr>0.5, 1, 0 )
  error <- sum(actual_results-prr!=0)/length(actual_results)
  #error <- compute_error( predicted_results, actual_results, type="rmse" )
  # return the error
  return(error)
}

# now let's check for k-nn

library(class)

data <- allL
data[11] <- as.factor(data[11]) # the label column

NUM_FOLDS <- 10
errors <- vector("numeric",NUM_FOLDS)  # container for errors
cvf <- cvFolds( nrow(data), K=NUM_FOLDS, type="consecutive" )
data <- data[sample(nrow(data)),]  # randomly permutes the rows
w <- cvf$which



for (i in 1:NUM_FOLDS)
{
  data.train <- data[w!=i,]  # training data
  data.notrain <- data[w==i,]   # non-training data
  # not really necessary, but safe...
  data.test <- data.notrain[,-11]   # removes the predicted column
  
  actual_results <- data.notrain$label
  error <- model_and_check_knn( data.train, data.test, actual_results, nc=nc )
  
  errors[i] <- error
  cat("error at fold ", i, " is ", error, "\n")
}

# plot the errors created
plot.new()
plot( errors, pch=19, col=3 ) # 3 for knn




#
# find best number of neighbors...
#



# run the cross-validation - species!
nc_errors <- vector("numeric",length(seq(1,15,2)))
nci <- 1
for (nc in seq(1,15,2))
{
errors <- vector("numeric",NUM_FOLDS)  # container for errors
for (i in 1:NUM_FOLDS)
{
  data.train <- data[w!=i,]  # training data
  data.notrain <- data[w==i,]   # non-training data
  # not really necessary, but safe...
  data.test <- data.notrain[,-11]   # removes the predicted column
  
  actual_results <- data.notrain$label
  error <- model_and_check_knn( data.train, data.test, actual_results, nc=nc )
  
  errors[i] <- error
  #cat("error at fold ", i, " is ", error, "\n")
}

# plot the errors created
#plot.new()
#plot( errors, pch=19, col=3 ) # 3 for knn
nc_errors[nci] <- mean(errors)
cat("for nc=", nc, " error: ", nc_errors[nci], "\n")
nci <- nci+1
}
plot(seq(1,15,2), nc_errors, pch=8, col=4)


# here is a model_and_check function for k-nn
model_and_check_knn <- function( data.train, data.test, actual_results, nc=5 )
{
  #library("neuralnet")
  # build the model
  predicted_results <- knn(data.train[,-11], data.test, data.train[,11], k=nc)
  # compute the error from predicted to actual
  table(predicted_results, actual_results)
  error <- sum(predicted_results != actual_results)/length(predicted_results)
  # return the error
  return(error)
}



